-- Modified canvas functions
gr	= love.graphics
local Canvas = {}
Canvas.__index = Canvas

function Canvas.Push(image)
	Canvas.image = gr.newImage( gr.newScreenshot() )
	Canvas.color = { love.graphics.getBackgroundColor() }
	gr.clear()
	gr.setColor(255,255,255,255)
	gr.draw(image)
	end

function Canvas:Pop()
	gr.setBackgroundColor( Canvas.color )
	gr.clear()
	gr.draw(Canvas.image)
	end

function Canvas.load( w, h )
	local data = love.image.newImageData( w, h )
	data:paste( gr.newScreenshot(), 0, 0, 0, 0, w, h )
	return gr.newImage( data )
	end


function newCanvas( v1, v2 )
	if type(v1) == 'string' then v1 = gr.newImage(v1) end
	if type(v1) == 'userdata' then
		gr.setColor( 255, 255, 255, 255 )
		Canvas.Push( v1 )
		canvas=Canvas.load(v1:getWidth(), v1:getHeight())
		Canvas:Pop()
		else
			local data=love.image.newImageData( v1,v2)
			img=gr.newImage(data)
			Canvas.Push( img )
			canvas = Canvas.load( v1, v2 )
			Canvas:Pop()
		end

	local tab = { canvas = canvas }
	setmetatable( tab, Canvas )
	return tab
	end

function Canvas:line( ... )
	Canvas.Push(self.canvas)
	local vert = {...}
	if type(vert[1]) == 'table' then vert=vert[1] end

	gr.setColor(Canvas.currentColor)
	for i = 1, #vert - 2, 2 do
		gr.line( vert[i], vert[i+1], vert[i+2], vert[i+3] )
		end
	self.canvas = Canvas.load(self.canvas:getWidth(),self.canvas:getHeight())
	Canvas:Pop()
	end

function Canvas:box( ... )
	Canvas.Push(self.canvas)
	gr.setColor(Canvas.currentColor)
	gr.rectangle( ... )
	self.canvas = Canvas.load(self.canvas:getWidth(),self.canvas:getHeight())
	Canvas:Pop()
	end

function Canvas:circle( ... )
	Canvas.Push(self.canvas)
	gr.setColor(Canvas.currentColor)
	gr.circle( ... )
	self.canvas = Canvas.load(self.canvas:getWidth(),self.canvas:getHeight())
	Canvas:Pop()
	end

function Canvas:arc( ... )
	Canvas.Push(self.canvas)
	gr.setColor(Canvas.currentColor)
	gr.arc( ... )
	self.canvas = Canvas.load(self.canvas:getWidth(),self.canvas:getHeight())
	Canvas:Pop()
	end

function Canvas:text( ... )
	Canvas.Push(self.canvas)
	gr.setColor(Canvas.currentColor)
	gr.print( ... )
	self.canvas = Canvas.load(self.canvas:getWidth(),self.canvas:getHeight())
	Canvas:Pop()
	end

function Canvas:point( x, y )
	Canvas.Push(self.canvas)
	gr.setColor(Canvas.currentColor)
	gr.point( x + 0.5, y + 0.5 )
	gr.point( x + 0.5, y - 0.5 )
	gr.point( x - 0.5, y - 0.5 )
	gr.point( x - 0.5, y + 0.5 )
	self.canvas = Canvas.load(self.canvas:getWidth(),self.canvas:getHeight())
	Canvas:Pop()
	end

function Canvas:polygon( ... )
	Canvas.Push(self.canvas)
	gr.setColor(Canvas.currentColor)
	gr.polygon( ... )
	self.canvas = Canvas.load(self.canvas:getWidth(),self.canvas:getHeight())
	Canvas:Pop()
	end

function Canvas:insertImage( ... )
	Canvas.Push(self.canvas)
	gr.setColor(255,255,255,255)
	gr.draw( ... )
	self.canvas = Canvas.load(self.canvas:getWidth(),self.canvas:getHeight())
	Canvas:Pop()
	end

function Canvas:triangle( ... )
	Canvas.Push(self.canvas)
	gr.setColor(Canvas.currentColor)
	gr.triangle( ... )
	self.canvas = Canvas.load(self.canvas:getWidth(),self.canvas:getHeight())
	Canvas:Pop()
	end

function Canvas:trianglestrip( mode, tab )
	for i = 1, #tab-2 do
		self:triangle( mode, tab[i][1],tab[i][2],tab[i+1][1],tab[i+1][2],tab[i+2][1],tab[i+2][2])
		end
	end

function Canvas:BackgroundColor( ... )
	self:Color( ... )
	self:box('fill',0,0,self.canvas:getWidth(),self.canvas:getHeight())
	end

function Canvas:Color( ... )
	Canvas.currentColor={...}
	end

function Canvas:draw( ... )
	arg = { ... }
	if type( arg[1] ) == 'userdata' then
			gr.setColor( 255, 255, 255, 255 )
			gr.draw( ... )
		else
		gr.draw(self.canvas, ...)
		end
	end

function Canvas:fontsize( num )
	gr.setFont( gr.newFont( num ) )
	end

return Canvas
